# Honeybee: A Plugin for Environmental Analysis (GPL)
# This file is part of Honeybee.
#
# Copyright (c) 2021, Ladybug Tools.
# You should have received a copy of the GNU General Public License
# along with Honeybee; If not, see <http://www.gnu.org/licenses/>.
# 
# @license GPL-3.0+ <http://spdx.org/licenses/GPL-3.0+>

"""
Search for available Mofidier Sets within the honeybee standards library.
-

    Args:
        keywords_: Optional keywords to be used to narrow down the output list of
            modifier sets. If nothing is input here, all available modifier sets
            will be output.
        join_words_: If False or None, this component will automatically split
            any strings of multiple keywords (spearated by spaces) into separate
            keywords for searching. This results in a greater liklihood of
            finding an item in the search but it may not be appropropriate for
            all cases. You may want to set it to True when you are searching for
            a specific phrase that includes spaces. Default: False.
    
    Returns:
        mod_sets: A list of modifier sets within the honeybee radiance
            standards library (filtered by keywords_ if they are input).
"""

ghenv.Component.Name = 'HB Search Mofidier Sets'
ghenv.Component.NickName = 'SearchModSets'
ghenv.Component.Message = '1.2.0'
ghenv.Component.Category = 'HB-Radiance'
ghenv.Component.SubCategory = '1 :: Modifiers'
ghenv.Component.AdditionalHelpFromDocStrings = '1'

try:  # import the honeybee-core dependencies
    from honeybee.search import filter_array_by_keywords
except ImportError as e:
    raise ImportError('\nFailed to import honeybee:\n\t{}'.format(e))

try:  # import the honeybee-radiance dependencies
    from honeybee_radiance.lib.modifiersets import MODIFIER_SETS
except ImportError as e:
    raise ImportError('\nFailed to import honeybee_radiance:\n\t{}'.format(e))


if len(keywords_) == 0:
    mod_sets = sorted(MODIFIER_SETS)
else:
    split_words = True if join_words_ is None else not join_words_
    mod_sets = sorted(filter_array_by_keywords(MODIFIER_SETS, keywords_, split_words))



# Honeybee: A Plugin for Environmental Analysis (GPL)
# This file is part of Honeybee.
#
# Copyright (c) 2021, Ladybug Tools.
# You should have received a copy of the GNU General Public License
# along with Honeybee; If not, see <http://www.gnu.org/licenses/>.
# 
# @license GPL-3.0+ <http://spdx.org/licenses/GPL-3.0+>

"""
Create a Bidirectional Scattering Distribution Function (BSDF) radiance modifier
from an XML file.
-

    Args:
        _xml_file: Path to an XML file contining BSDF data. These files can
            be produced using the LBNL WINDOW software among other sources.
        _up_vec_: A vector that sets the hemisphere that the BSDF modifier faces.
            For materials that are symmetrical about the face plane (like
            non-angled venetian blinds), this can be any vector that is not
            perfectly normal/perpendicular to the face. For asymmetrica
            materials like angled venetian blinds, this variable should be
            coordinated with the direction that the geometry is facing.
            The default is set to (0.01, 0.01, 1.00), which should hopefully
            not be normal to any typical face.
        thickness_: Optional number to set the thickness of the BSDF. (Default: 0).

    Returns:
        modifier: A BSDF modifier that can be assigned to a Honeybee geometry
            or Modifier Sets.
"""

ghenv.Component.Name = 'HB BSDF Modifier'
ghenv.Component.NickName = 'BSDFMod'
ghenv.Component.Message = '1.2.0'
ghenv.Component.Category = 'HB-Radiance'
ghenv.Component.SubCategory = '1 :: Modifiers'
ghenv.Component.AdditionalHelpFromDocStrings = '2'

try:  # import the honeybee-radiance dependencies
    from honeybee_radiance.modifier.material import BSDF
except ImportError as e:
    raise ImportError('\nFailed to import honeybee_radiance:\n\t{}'.format(e))

try:  # import ladybug_rhino dependencies
    from ladybug_rhino.grasshopper import all_required_inputs
except ImportError as e:
    raise ImportError('\nFailed to import ladybug_rhino:\n\t{}'.format(e))


if all_required_inputs(ghenv.Component):
    # process the vector input
    if _up_vec_ is not None:
        _up_vec_ = (_up_vec_.X, _up_vec_.Y, _up_vec_.Z)

    # create the modifier
    modifier = BSDF(
        _xml_file, up_orientation=_up_vec_, thickness=thickness_)



# This file is part of Honeybee.
#
# Copyright (c) 2021, Ladybug Tools.
# You should have received a copy of the GNU General Public License
# along with Honeybee; If not, see <http://www.gnu.org/licenses/>.
# 
# @license GPL-3.0+ <http://spdx.org/licenses/GPL-3.0+>

"""
Convert a High Dynamic Range (HDR) image file into a falsecolor version of itself.
-

    Args:
        _hdr: Path to a High Dynamic Range (HDR) image file.
        legend_unit_: Text for the unit of the legend. If unspecified, an attempt will
            be made to sense the metric from the input image file. Typical examples
            include lux, W/m2, cd/m2, w/sr-m2.
        conversion_: Number for the conversion factor (aka. multiplier) for the results.
            The default is either 1 or 179 depending on whether the image is for
            radiance or irradiance to luminance or illuminance, respectively.
        max_: A number to set the upper boundary of the legend. The default is
            dictated based on the legend_unit_.
        seg_count_: An interger representing the number of steps between the
            high and low boundary of the legend. The default is set to 10
            and any custom values input in here should always be greater
            than or equal to 2.
        contour_lines_: Set to True ro render the image with colored contour lines.
        extrema_: Set to True to cause extrema points to be printed on the brightest
            and darkest pixels of the input picture.
        color_palette_: Optional interger or text to change the color palette.
            Choose from the following.
                * 0 = def - default colors
                * 1 = pm3d -  a variation of the default colors
                * 2 = spec - the old spectral mapping
                * 3 = hot - a thermal scale

    Returns:
        hdr: Path to the resulting falsecolor HDR file. This can be plugged into the
            Ladybug "Image Viewer" component to preview the image. It can also
            be plugged into the "HB HDR to GIF" component to get a GIF image
            that is more portable and easily previewed by different software.
"""

ghenv.Component.Name = 'HB False Color'
ghenv.Component.NickName = 'FalseColor'
ghenv.Component.Message = '1.2.0'
ghenv.Component.Category = 'HB-Radiance'
ghenv.Component.SubCategory = '4 :: Results'
ghenv.Component.AdditionalHelpFromDocStrings = '2'

import os

try:  # import honeybee_radiance_command dependencies
    from honeybee_radiance_command.falsecolor import Falsecolor
except ImportError as e:
    raise ImportError('\nFailed to import honeybee_radiance_command:\n\t{}'.format(e))

try:  # import honeybee_radiance dependencies
    from honeybee_radiance.config import folders as rad_folders
except ImportError as e:
    raise ImportError('\nFailed to import honeybee_radiance:\n\t{}'.format(e))

try:  # import ladybug_rhino dependencies
    from ladybug_rhino.grasshopper import all_required_inputs
except ImportError as e:
    raise ImportError('\nFailed to import ladybug_rhino:\n\t{}'.format(e))

# check the Radiance date of the installed radiance
try:  # import lbt_recipes dependencies
    from lbt_recipes.version import check_radiance_date
except ImportError as e:
    raise ImportError('\nFailed to import lbt_recipes:\n\t{}'.format(e))
check_radiance_date()


def sense_metric_from_hdr(hdr_path):
    """Sense the metric/units of a given HDR file from its properties.

    Args:
        hdr_path: The path to an HDR image file

    Returns:
        Text for the units of the file (either 'lux', 'W/m2', 'cd/m2', 'W/sr-m2')
    """
    with open(hdr_path, 'r') as hdr_file:
        for lineCount, line in enumerate(hdr_file):
            if lineCount < 10:
                low_line = line.strip().lower()
                if low_line.startswith('oconv') and low_line.endswith('.sky'):
                        return 'W/sr-m2'  # this is an image of a sky
                if low_line.startswith('rpict'):
                    if line.find('-i') > -1:
                        return 'lux'
            else:  # we have passed the header of the file
                return 'cd/m2'  # luminance



if all_required_inputs(ghenv.Component):
    # set up the paths for the various files used in translation
    img_dir = os.path.dirname(_hdr)
    input_image = os.path.basename(_hdr)
    new_image = input_image.lower().replace('.hdr', '_falsecolor.HDR')
    hdr = os.path.join(img_dir, new_image)

    # set default properties
    seg_count_ = seg_count_ if seg_count_ is not None else 10
    if legend_unit_ is None:
        legend_unit_ = sense_metric_from_hdr(_hdr)
    if max_ is None:
        if legend_unit_ in ('W/sr-m2', 'W/m2'):
            max_ = '200'
        else:
            max_ = '20000'
    if conversion_ is None:
        if legend_unit_ == 'W/m2':
            conversion_ = 1
        else:
            conversion_ = 179

    # create the command to run falsecolor
    falsecolor = Falsecolor(input=input_image, output=new_image)
    falsecolor.options.s = max_
    falsecolor.options.n = seg_count_
    falsecolor.options.l = legend_unit_
    falsecolor.options.m = conversion_
    if contour_lines_:
        falsecolor.options.cl = True
        falsecolor.options.p = input_image
    if extrema_:
        falsecolor.options.e = True
    if color_palette_:
        PALETTE_DICT = {
            '0': 'def',
            '1': 'pm3d',
            '2': 'spec',
            '3': 'hot',
            'def': 'def',
            'pm3d': 'pm3d',
            'spec': 'spec',
            'hot': 'hot'
        }
        falsecolor.options.pal = PALETTE_DICT[color_palette_]

    # run the falsecolor command
    env = None
    if rad_folders.env != {}:
        env = rad_folders.env
    env = dict(os.environ, **env) if env else None
    falsecolor.run(env, cwd=img_dir)



# Honeybee: A Plugin for Environmental Analysis (GPL) started by Mostapha Sadeghipour Roudsari
# This file is part of Honeybee.
#
# You should have received a copy of the GNU General Public License
# along with Honeybee; If not, see <http://www.gnu.org/licenses/>.
# 
# @license GPL-3.0+ <http://spdx.org/licenses/GPL-3.0+>

"""
Annual Daylight Metrics.

-

    Args:
        _results: An list of annual Radiance result files from the "HB Annual Radiation"
            component.  This should include both the .ill files and the
            sun-up-hours.txt
        hoys_: An optional integer or list of integers (each greater than or equal to 0)
            to select the hours of the year (HOYs) for which radiation results
            will be displayed. These HOYs can be obtained from the "LB Calculate
            HOY" or the "LB Analysis Period" components. If None, all hours of
            the results will be used.
        timestep_: The timesteps per hour of the Wea that was used for the radiation analysis.
            This will be used to ensure radiation values are in the correct
            units. (Default: 1).

    Returns:
        report: Reports, errors, warnings, etc.
        irradiance: Average irradiance valules for each sensor in W/m2.
        radiation: Cumulative radiation valules for each sensor in Wh/m2.
"""

ghenv.Component.Name = "HB Annual Radiation Result"
ghenv.Component.NickName = 'RadiationResult'
ghenv.Component.Message = '1.2.0'
ghenv.Component.Category = 'HB-Radiance'
ghenv.Component.SubCategory = '4 :: Results'
ghenv.Component.AdditionalHelpFromDocStrings = '1'

try:
    from ladybug_rhino.grasshopper import all_required_inputs, list_to_data_tree
except ImportError as e:
    raise ImportError('\nFailed to import ladybug_rhino:\n\t{}'.format(e))


def parse_sun_up_hours(result_files, hoys, timestep):
    """Parse the sun-up hours from the result file .txt file.

    Args:
        result_files: A list of result files that contains the .txt file.
        hoys: A list of 8760 * timestep values for the hoys to select. If an empty
            list is passed, None will be returned.
    """
    for i, r_file in enumerate(result_files):
        if r_file.endswith('.txt'):
            result_files.pop(i)  # remove it from the list
            if len(hoys) != 0:
                schedule = [False] * (8760 * timestep)
                for hr in hoys:
                    schedule[int(hr * timestep)] = True
                with open(r_file) as soh_f:
                    occ_pattern = [schedule[int(float(h) * timestep)] for h in soh_f]
                return occ_pattern


def cumulative_radiation(ill_file, occ_pattern, timestep):
    """Compute cumulative radiation for a given result file."""
    irradiance, radiation = [], []
    with open(ill_file) as results:
        if occ_pattern is None:  # no HOY filter on results
            for pt_res in results:
                values = [float(r) for r in pt_res.split()]
                total_val = sum(values)
                irradiance.append(total_val / len(values))
                radiation.append(total_val / timestep)
        else: 
            for pt_res in results:
                values = [float(r) for r, is_hoy in zip(pt_res.split(), occ_pattern) if is_hoy]
                total_val = sum(values)
                irradiance.append(total_val / len(values))
                radiation.append(total_val / timestep)
    return irradiance, radiation


if all_required_inputs(ghenv.Component):
    # process the sun-up hours and parse timestep
    timestep_ = 1 if timestep_ is None else timestep_
    occ_pattern = parse_sun_up_hours(_results, hoys_, timestep_)

    # compute the annual metrics
    irradiance, radiation = [], []
    for ill_file in _results:
        irr, rad = cumulative_radiation(ill_file, occ_pattern, timestep_)
        irradiance.append(irr)
        radiation.append(rad)
    irradiance = list_to_data_tree(irradiance)
    radiation = list_to_data_tree(radiation)



# This file is part of Honeybee.
#
# Copyright (c) 2021, Ladybug Tools.
# You should have received a copy of the GNU General Public License
# along with Honeybee; If not, see <http://www.gnu.org/licenses/>.
# 
# @license GPL-3.0+ <http://spdx.org/licenses/GPL-3.0+>

"""
Convert a High Dynamic Range (HDR) image file into a Graphics Interchange Format
(GIF) image file.
_
GIF files are much smaller than HDRs, they are more portable, and they can be
previewed with many different types of software. However, they do not contain
all of the information that an HRD has.
-

    Args:
        _hdr: Path to a High Dynamic Range (HDR) image file.
        adj_expos_: Boolean to note whether the exposuer of the image shoudl be adjusted to
            mimic the human visual response in the output. The goal of this process
            is to output an image that correlates more strongly with a person’s
            subjective impression of a scene rather than the absolute birghtness
            of the scene.

    Returns:
        gif: Path to the resulting GIF file,
"""

ghenv.Component.Name = 'HB HDR to GIF'
ghenv.Component.NickName = 'HDR-GIF'
ghenv.Component.Message = '1.2.0'
ghenv.Component.Category = 'HB-Radiance'
ghenv.Component.SubCategory = '4 :: Results'
ghenv.Component.AdditionalHelpFromDocStrings = '2'

import os

try:  # import honeybee_radiance_command dependencies
    from honeybee_radiance_command.pcond import Pcond
    from honeybee_radiance_command.ra_gif import Ra_GIF
except ImportError as e:
    raise ImportError('\nFailed to import honeybee_radiance_command:\n\t{}'.format(e))

try:  # import honeybee_radiance dependencies
    from honeybee_radiance.config import folders as rad_folders
except ImportError as e:
    raise ImportError('\nFailed to import honeybee_radiance:\n\t{}'.format(e))

try:  # import ladybug_rhino dependencies
    from ladybug_rhino.grasshopper import all_required_inputs
except ImportError as e:
    raise ImportError('\nFailed to import ladybug_rhino:\n\t{}'.format(e))

# check the Radiance date of the installed radiance
try:  # import lbt_recipes dependencies
    from lbt_recipes.version import check_radiance_date
except ImportError as e:
    raise ImportError('\nFailed to import lbt_recipes:\n\t{}'.format(e))
check_radiance_date()


if all_required_inputs(ghenv.Component):
    # set up the paths for the various files used in translation
    img_dir = os.path.dirname(_hdr)
    input_image = os.path.basename(_hdr)
    new_image = input_image.lower().replace('.hdr', '.gif')
    gif = os.path.join(img_dir, new_image)

    # create the command to run the conversion to GIF
    if adj_expos_:
        adj_image = input_image.lower().replace('.hdr', '_h.HDR')
        pcond = Pcond(input=raw_hdr, output=adj_image)
        pcond.options.h = True
        ra_gif = Ra_GIF(input=adj_image, output=new_image)
        commands = (pcond, ra_gif)
    else:
        ra_gif = Ra_GIF(input=input_image, output=new_image)
        commands = (ra_gif,)

    # run the commands in series and load the global horizontal irradiance
    env = None
    if rad_folders.env != {}:
        env = rad_folders.env
    env = dict(os.environ, **env) if env else None
    for r_cmd in commands:
        r_cmd.run(env, cwd=img_dir)



# Honeybee: A Plugin for Environmental Analysis (GPL) started by Mostapha Sadeghipour Roudsari
# This file is part of Honeybee.
#
# You should have received a copy of the GNU General Public License
# along with Honeybee; If not, see <http://www.gnu.org/licenses/>.
# 
# @license GPL-3.0+ <http://spdx.org/licenses/GPL-3.0+>

"""
Create a State object representing a single dynamic group state.
-

    Args:
        modifier_: A Honeybee Radiance Modifier object to be applied to this state's
            parent in this state. This is used to swap out the modifier in
            multi-phase studies. If None, it will be the parent's default modifier.
        shades_: An optional array of StateGeometry objects to be included
            with this state.
    
    Returns:
        state: A Honeybee State object representing a single dynamic group state.
            This can be assigned to apertures or shades using the "HB Dynamic
            Aperture Group" componet or the "HB Dynamic Shade Group" component.
"""

ghenv.Component.Name = 'HB Dynamic State'
ghenv.Component.NickName = 'State'
ghenv.Component.Message = '1.2.0'
ghenv.Component.Category = 'HB-Radiance'
ghenv.Component.SubCategory = '0 :: Basic Properties'
ghenv.Component.AdditionalHelpFromDocStrings = '3'

try:
    from honeybee_radiance.dynamic import RadianceSubFaceState
except ImportError as e:
    raise ImportError('\nFailed to import honeybee_radiance:\n\t{}'.format(e))

state = RadianceSubFaceState(modifier_, [geo.duplicate() for geo in shades_])
